`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
//
// Date:         2011-10-14
// Module Name:  hello_world.v
// Engineer:     Scott Gravenhorst
// Description:  A series of hello world projects regarding the wishbone bus and
//               the on board SDRAM.
// Version:      0.00
//
// This project is based on the SDRAM interface found in Rick Huang's widebus 
// recorder project.  Attribution is found below
//
// Reformat by Scott Gravenhorst
//
// This project runs at a clock rate of 100 MHz.  2 DCMs are used, the first 
// provides 2X clock out from 50 MHz input.  The second DCM receives the first DCM
// CLKX2 output (100 MHz) and generates the phases required by the project (mainly by the
// SDRAM interface).
//
// ver_a: As of 2011-10-16, this project works.
//  NOTE: The reset system requires that you hold down the rotary encoder push
//        button and press and release the south button and then release the
//        rotary encoder pushbutton.  Failure to do so will cause unpredictable
//        results (the LEDs will not do what they are supposed to do).
//
// ver_b: Attempt to add an automatic power on reset.
//        Power on 202 uSec reset.
//        Tests indicate that 200 minimum uSec reset is required.  This allows
//        the interface hardware and DDR SDRAM to initialize.
//        Works as of 2011-10-16
//
// ver_c: tighten main state machine in ctrl_FSM
//
// ver_d: Use wishbone DAC, play from block RAM.  Block RAM will be initialized
//        with a complete sinewave (4 block RAMs).
//
// ver_e: Copy block RAM to SDRAM and play to DAC.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Original Comments Header:

////////////////////////////////////////////////////////////////////////////////
//
// Date:       Feb. 27, 2007
// Module Name:    Base.v
// Description:    Main wiring block for the recoder project
//
// Revision:
// Revision 0.01 - File Created
//
// File format:    This file has been formmated to use tabstop of 4
//
// User Interface:
//       Button:
//       South  = Stop/reset
//       North  = Clock reset (Usually not required)
//       East   = Play
//       West   = Record
//       Center = Clear memory
//
//       Sample rate and input gain is adjusted in the recoder control
//       block.
//
// Development platform: Spartan-3E Starter kit
//
// Copyright (C) 2007, Rick Huang
//   
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// 
////////////////////////////////////////////////////////////////////////////////

module SDRAM_interface( 
      clk,       clk_n,      clk_90,      clk_270,
//      btn_north, 
      btn_south, 
//      rot_cnt,
      sd_A, 
      sd_DQ, 
      sd_BA,
      sd_CAS, 
      sd_CK_N, 
      sd_CK_P, 
      sd_CKE, 
      sd_CS, 
      sd_LDM, sd_LDQS, sd_RAS, 
      sd_UDM, sd_UDQS, sd_WE, 
      power_on,
      GO,                       // Go do the SDRAM operation
      RW,                       // Read or Write command flag
      USER_WRITE_DATA,          // input from user, the data he wants written
      USER_READ_DATA,           // output to be read by user.  This is his requested read data.
      USER_RAM_ADDR,            // User RAM Address.  Absolute address, not wishboned
      USER_READY,               // State of SDRAM state machine.  0 = operation pending, 1 = ready for new command
      led
      );

  input         clk;
  input         clk_n;
  input         clk_90;
  input         clk_270;

//  input         btn_north;
//  input         btn_east;
  input         btn_south;
//  input         btn_west;
//  input         rot_cnt;

  // DDR-SDRAM connection
  output [12:0] sd_A;
  inout  [15:0] sd_DQ;
  output [1:0]  sd_BA;
  output        sd_CAS;
  output        sd_CK_N, sd_CK_P;
  output        sd_CKE;
  output        sd_CS;
  output        sd_LDM, sd_UDM;
  inout         sd_LDQS, sd_UDQS;
  output        sd_RAS;
  output        sd_WE;
//  inout         sd_dbg_st;
  
  // USER CONTROL CONNECTIONS:
  input         GO;
  input         RW;
  input  [31:0] USER_WRITE_DATA;
  output [31:0] USER_READ_DATA;
  input  [31:0] USER_RAM_ADDR;
  output        USER_READY; 

  output        power_on;  

  output  [7:0] led;
  // Debug storage
//  reg  [31:0]   debug0;

  wire         GO;
  wire         RW;
  wire  [31:0] USER_WRITE_DATA;
  wire  [31:0] USER_READ_DATA;
  wire  [31:0] USER_RAM_ADDR;
  wire         USER_READY; 
  
  // Wishbone bus nets - Ref on MASTER side //
  wire [31:0]   wADR_O;
  wire [31:0]   wDAT_O;
  wire [31:0]   wDAT_I;
  wire          wSTB_O;
  wire          wWE_O;
  wire          wACK_I;
  wire          spi_sdo;
  wire          ext_addr_STB;
//  reg  [31:0]   reg_read_buf;

  // ** Invoke the functional block **********************************
  // synchronize reset
  //
  wire clk;
  wire clk_n;
  wire clk_90;
  wire clk_270;

  
  // ** DDR-SDRAM I/O block, map to wishbone adddress 0x20000000 ******
  //
  wire [31:0] wSD_DAT_I;
  wire wSD_ack;
  wire wSD_STB_O;
  
  wire sd_CLK_O;

  assign sd_CK_P = clk_n;
  assign sd_CK_N = clk;
  
  ddr_sdram32 m_sd
    (
    .sd_A_O(sd_A), .sd_BA_O(sd_BA), .sd_D_IO(sd_DQ), 
    .sd_RAS_O(sd_RAS), .sd_CAS_O(sd_CAS), 
    .sd_WE_O(sd_WE),
    .sd_UDM_O(sd_UDM), .sd_LDM_O(sd_LDM), 
    .sd_UDQS_IO(sd_UDQS), .sd_LDQS_IO(sd_LDQS),
    .sd_CS_O(sd_CS), .sd_CKE_O(sd_CKE), 
    .sd_CLK_O(),
    .main_clk(clk), .main_clk_n(clk_n),
    .clk90(clk_90), .clk270(clk_270),
    .wADR_I(wADR_O[25:0]), .wSTB_I(wSD_STB_O),
    .wDAT_I(wDAT_O), .wDAT_O(wSD_DAT_I),
    .wACK_O(wSD_ack), 
    .wWE_I(wWE_O),
    .reset_I( btn_south ),                          // BUTTON SOUTH RESET
    .wBURST_I( 1'b0 )
    );

  assign wSD_STB_O = (wADR_O[31:28] == 4'h2) ? wSTB_O : 0;   // wishbone address decode
  
  // ** SDRAM read and write state machine ***************************************
  // 

  SDRAM_ctrl_FSM ctrl1 
    (
    .reset( btn_south ),         // BUTTON SOUTH RESET 
    .main_clk( clk ),                  
    .wADR_O( wADR_O ), 
    .wSTB_O( wSTB_O ), 
    .wDAT_O( wDAT_O ), 
    .wDAT_I( wDAT_I ),
    .wACK_I( wACK_I ), 
    .wWE_O( wWE_O ),
    .RW( RW ),                                      // user Read or Write request
    .USER_GO( GO ),                                 // user GO bit
    .USER_WRITE_DATA( USER_WRITE_DATA ),
    .USER_READ_DATA( USER_READ_DATA ),
    .USER_RAM_ADDR( USER_RAM_ADDR ),
    .USER_READY( USER_READY ),
    .power_on( power_on ),         // provides 200 uSec delay for SDRAM initialization
    .led( led )                    // led output for diagnostics
    );

  // Use ORed bus structure
//  assign wACK_I = wLED_ack   | wSD_ack | wADC_ack;
//  assign wDAT_I = wLED_DAT_I | wSD_DAT_I | wADC_DAT_I;
  assign wACK_I = wSD_ack;     // don't have LED and ADC
  assign wDAT_I = wSD_DAT_I;

  endmodule
  